#!/usr/bin/python

"""
RO manager parameter settings
"""

import __init__
VERSION_NUM   = __init__.__version__

RO_VERSION    = "v"+VERSION_NUM
VERSION       = RO_VERSION+" (RODL)"
MANIFEST_DIR  = ".ro"
MANIFEST_FILE = "manifest.rdf"

# End.
