# __init__.py
#__version__ = "0.2.1"       # Initial version with installation package
#__version__ = "0.2.2"       # Updated README documentation for PyPI page
#__version__ = "0.2.3"       # Experimenting with distribution options
#__version__ = "0.2.4"       # Added MANIFEST.in so that data files are part of sdist
__version__ = "0.2.5"       # Drop references to lpod-show from test suite
